=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-21 03:37:53 +0000
+++ bzrlib/builtins.py	2006-07-27 18:18:22 +0000
@@ -1171,9 +1171,14 @@
                 raise BzrCommandError("Files are in different branches")
             file_list = None
         except NotBranchError:
-            # Don't raise an error when bzr diff is called from
-            # outside a working tree.
-            tree1, tree2 = None, None
+            if (revision is not None and len(revision) == 2
+                and not revision[0].needs_branch()
+                and not revision[1].needs_branch()):
+                # If both revision specs include a branch, we can
+                # diff them without needing a local working tree
+                tree1, tree2 = None, None
+            else:
+                raise
         if revision is not None:
             if tree2 is not None:
                 raise BzrCommandError("Can't specify -r with two branches")

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2006-07-21 20:32:04 +0000
+++ bzrlib/revisionspec.py	2006-07-27 18:14:48 +0000
@@ -170,7 +170,13 @@
         return '<%s %s%s>' % (self.__class__.__name__,
                               self.prefix or '',
                               self.spec)
+    
+    def needs_branch(self):
+        """Whether this revision spec needs a branch.
 
+        Set this to False the branch argument of _match_on is not used.
+        """
+        return True
 
 # private API
 
@@ -208,6 +214,9 @@
                 return RevisionInfo(other_branch, None)
             revid = other_branch.get_rev_id(revno)
             return RevisionInfo(other_branch, revno)
+        
+    def needs_branch(self):
+        return self.spec.find(':') == -1
 
 SPEC_TYPES.append(RevisionSpec_revno)
 

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2006-07-08 20:38:36 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2006-07-27 04:53:20 +0000
@@ -146,7 +146,7 @@
                           "-new content\n"
                           "+contents of branch1/file\n"
                           "\n", subst_dates(out))
-        out, ett = self.run_bzr_captured(['diff', 'branch2', 'branch1'],
+        out, err = self.run_bzr_captured(['diff', 'branch2', 'branch1'],
                                          retcode=1)
         self.assertEquals('', err)
         self.assertEqualDiff("=== modified file 'file'\n"
@@ -160,13 +160,16 @@
     def test_diff_revno_branches(self):
         self.example_branches()
         print >> open('branch2/file', 'wb'), 'even newer content'
-        self.run_bzr_captured(['commit', '-m', 'update file once more', 'branch2'])
+        self.run_bzr_captured(['commit', '-m', 
+                               'update file once more', 'branch2'])
 
-        out, err = self.run_bzr_captured(['diff', '-r', 'revno:1:branch2..revno:1:branch1'],
+        out, err = self.run_bzr_captured(['diff', '-r',
+                                          'revno:1:branch2..revno:1:branch1'],
                                          retcode=0)
         self.assertEquals('', err)
         self.assertEquals('', out)
-        out, ett = self.run_bzr_captured(['diff', '-r', 'revno:2:branch2..revno:1:branch1'],
+        out, err = self.run_bzr_captured(['diff', '-r', 
+                                          'revno:2:branch2..revno:1:branch1'],
                                          retcode=1)
         self.assertEquals('', err)
         self.assertEqualDiff("=== modified file 'file'\n"
@@ -191,7 +194,8 @@
         self.example_branch2()
         
         print >> open('branch1/file1', 'wb'), 'new line'
-        output = self.run_bzr_captured(['diff', '-r', '1..', 'branch1'], retcode=1)
+        output = self.run_bzr_captured(['diff', '-r', '1..', 'branch1'],
+                                       retcode=1)
         self.assertTrue('\n-original line\n+new line\n' in output[0])
 
     def test_diff_across_rename(self):

=== modified file 'bzrlib/tests/blackbox/test_outside_wt.py'
--- bzrlib/tests/blackbox/test_outside_wt.py	2006-06-05 17:23:03 +0000
+++ bzrlib/tests/blackbox/test_outside_wt.py	2006-07-27 04:58:47 +0000
@@ -41,3 +41,38 @@
                                 url, retcode=3)
         self.assertEqual(u'bzr: ERROR: Not a branch:'
                          u' %s\n' % url, err)
+
+    def test_diff_ouside_tree(self):
+        os.chdir(tempfile.mkdtemp())
+        self.run_bzr_captured(['init', 'branch1'])
+        self.run_bzr_captured(['commit', '-m', 'nothing', 
+                               '--unchanged', 'branch1'])
+        self.run_bzr_captured(['commit', '-m', 'nothing', 
+                               '--unchanged', 'branch1'])
+        # -r X..Y
+        out, err = self.run_bzr_captured(['diff', 
+                                          '-r', 'revno:2:branch2..revno:1'],
+                                         retcode=3)
+        self.assertEquals('', out)
+        self.assertEqual(u'bzr: ERROR: Not a branch: %s/\n' % (getcwd(),),
+                         err)
+        # -r X
+        out, err = self.run_bzr_captured(['diff', '-r', 'revno:2:branch2'],
+                                         retcode=3)
+        self.assertEquals('', out)
+        self.assertEqual(u'bzr: ERROR: Not a branch: %s/\n' % (getcwd(),),
+                         err)
+        # -r X..
+        out, err = self.run_bzr_captured(['diff', '-r', 'revno:2:branch2..'],
+                                         retcode=3)
+        self.assertEquals('', out)
+        self.assertEqual(u'bzr: ERROR: Not a branch: %s/\n' % (getcwd(),),
+                         err)
+        # no -r at all.
+        out, err = self.run_bzr_captured(['diff'],
+                                         retcode=3)
+        self.assertEquals('', out)
+        self.assertEqual(u'bzr: ERROR: Not a branch: %s/\n' % (getcwd(),),
+                         err)
+        
+

